<?php
namespace nur\sery\schema;

class ref_schema {
  const SCHEMA_SCHEMA = [
    "type" => ["string", null, "" => 0,
      "required" => true,
      "desc" => "type de la valeur",
    ],
    "default" => [null, null, "" => 1,
      "desc" => "valeur par défaut",
    ],
    "key" => ["?key", null, "" => "",
      "desc" => "clé de la valeur si elle est dans un tableau",
    ],
    "required" => ["bool", false,
      "desc" => "cette valeur est-elle requise?",
    ],
    "formatter" => ["?callable", null,
      "desc" => "signature de la fonction: (value, format, type) => string",
    ],
    "parser" => ["?callable", null,
      "desc" => "signature de la fonction: (value, type) => [value, result]",
    ],
    "messages" => ["?array", null,
      "desc" => "message à afficher en cas d'erreur d'analyse",
    ],
  ];

  const MESSAGES = [
    "absent" => "{key}: Vous devez spécifier cette valeur",
    "null" => "{key}: Cette valeur ne doit pas être nulle",
    "empty" => "{key}: Cette valeur ne doit pas être vide",
    "invalid" => "{key}: {orig}: cette valeur est invalide",
  ];

  /** @var array schéma des natures de schéma */
  public const NATURE_METASCHEMA = [
    "nature" => ["string", null, "nature du schéma",
      "key" => 0,
      "allowed_values" => ["assoc", "list", "scalar"],
    ],
    "title" => ["?string", null, "libellé de la valeur"],
    "required" => ["bool", false, "la valeur est-elle requise?"],
    "nullable" => ["?bool", null, "la valeur peut-elle être nulle?"],
    "desc" => ["?content", null, "description de la valeur"],
    "name" => ["?key", null, "identifiant de la valeur"],
    "schema" => ["?array", null, "définition du schéma"],
  ];

  /** @var array meta-schema d'un schéma de nature scalaire */
  public const SCALAR_METASCHEMA = [
    "type" => ["array", null, "types possibles de la valeur", "required" => true],
    "default" => [null, null, "valeur par défaut si la valeur n'existe pas"],
    "title" => ["?string", null, "libellé de la valeur"],
    "required" => ["bool", false, "la valeur est-elle requise?"],
    "nullable" => ["?bool", null, "la valeur peut-elle être nulle?"],
    "desc" => ["?content", null, "description de la valeur"],
    "analyzer_func" => ["?callable", null, "fonction qui analyse une valeur entrante et indique comment la traiter"],
    "extractor_func" => ["?callable", null, "fonction qui extrait la valeur à analyser dans une chaine de caractère"],
    "parser_func" => ["?callable", null, "fonction qui analyse une chaine de caractères pour produire la valeur"],
    "normalizer_func" => ["?callable", null, "fonction qui normalise la valeur"],
    "messages" => ["?array", null, "messages à afficher en cas d'erreur d'analyse"],
    "formatter_func" => ["?callable", null, "fonction qui formatte la valeur pour affichage"],
    "format" => [null, null, "format à utiliser pour l'affichage"],
    "" => ["" => ["assoc", "schema" => self::NATURE_METASCHEMA]],
    "name" => ["?key", null, "identifiant de la valeur"],
  ];
}
